@@ -38,7 +38,7 @@ namespace euf {
 namespace arith {
 
     typedef ptr_vector<lp_api::bound<sat::literal>> lp_bounds;
-    typedef lp::var_index lpvar;
+    typedef lp::lpvar lpvar;
     typedef euf::theory_var theory_var;
     typedef euf::theory_id theory_id;
     typedef euf::enode enode;
@@ -245,7 +245,7 @@ namespace arith {
         symbol                       m_farkas;
         std_vector<lp::implied_bound> m_implied_bounds;
         lp::lp_bound_propagator<solver> m_bp;
-        mutable vector<std::pair<lp::tv, rational>> m_todo_terms;
+        mutable vector<std::pair<lp::lpvar, rational>> m_todo_terms;
 
         // lemmas
         lp::explanation     m_explanation;
@@ -306,7 +306,7 @@ namespace arith {
         bool reflect(expr* n) const;
 
         lpvar get_lpvar(theory_var v) const;
-        lp::tv get_tv(theory_var v) const;
+        lp::lpvar get_column(theory_var v) const;
 
         // axioms
         void mk_div_axiom(expr* p, expr* q);
@@ -348,7 +348,7 @@ namespace arith {
             iterator end,
             bool& found_compatible);
 
-        void propagate_eqs(lp::tv t, lp::constraint_index ci, lp::lconstraint_kind k, api_bound& b, rational const& value);
+        void propagate_eqs(lp::lpvar t, lp::constraint_index ci, lp::lconstraint_kind k, api_bound& b, rational const& value);
         void propagate_basic_bounds(unsigned qhead);
         void propagate_bounds_with_lp_solver();
         void propagate_bound(literal lit, api_bound& b);
@@ -362,9 +362,9 @@ namespace arith {
         api_bound* mk_var_bound(sat::literal lit, theory_var v, lp_api::bound_kind bk, rational const& bound);
         lp::lconstraint_kind bound2constraint_kind(bool is_int, lp_api::bound_kind bk, bool is_true);
         void fixed_var_eh(theory_var v1, u_dependency* dep, rational const& bound);
-        bool set_upper_bound(lp::tv t, lp::constraint_index ci, rational const& v) { return set_bound(t, ci, v, false); }
-        bool set_lower_bound(lp::tv t, lp::constraint_index ci, rational const& v) { return set_bound(t, ci, v, true); }
-        bool set_bound(lp::tv tv, lp::constraint_index ci, rational const& v, bool is_lower);
+        bool set_upper_bound(lp::lpvar t, lp::constraint_index ci, rational const& v) { return set_bound(t, ci, v, false); }
+        bool set_lower_bound(lp::lpvar t, lp::constraint_index ci, rational const& v) { return set_bound(t, ci, v, true); }
+        bool set_bound(lp::lpvar tv, lp::constraint_index ci, rational const& v, bool is_lower);
 
         typedef std::pair<lp::constraint_index, rational> constraint_bound;
         vector<constraint_bound>        m_lower_terms;
@@ -483,6 +483,7 @@ namespace arith {
         arith_proof_hint const* explain_conflict(hint_type ty, sat::literal_vector const& core, euf::enode_pair_vector const& eqs);
         void explain_assumptions(lp::explanation const& e);
 
+        bool validate_conflict();
 
     public:
         solver(euf::solver& ctx, theory_id id);
